
/******************************************************************
 *                                                                *
 *                  Computer Science Department                   *
 *                                                                *
 *                        Master's Project                        *
 *                                                                *
 *                Graphical Development Environment               *
 *                                                                *
 *                     for Postgres Database                      *
 *                                                                *
 *                    Designed Especially for                     *
 *                                                                *
 *                 Postgres Developers and Users                  *
 *                                                                *
 *                              At                                *
 *                                                                *
 *                     Concordia University                       *
 *                                                                *
 ******************************************************************
 *                                                                *
 * Designer and Programmer:  Khaled Jababo                        *
 * Email                  :  jababo@cs.concordia.ca               *
 *                                                                *
 * Supervised by          :  Dr. Bipin C. Desai.                  *
 *                           Computer Science Department.         *
 * Email                  :  bcdesai@cs.concordia.ca              *
 *                                                                *
 * Last Modified          :  June 18, 1994                        *
 *                                                                *
 *                                                                *
 *                                                                *
 * Disclaimer:  There are no guarantees as to the accuracy, cu-   *
 *             rrency, quality, or usefulness of these programs   *
 *             or any information that is provided by using it.   *
 *             Furthermore, there is no assumption of responsi-   *
 *             bility for any los or damage which may be caused   *
 *             by their use.                                      *
 *                                                                *
 *                                                                *
 * Forward comments or questions about this report or the GDEP    *
 * system to:                                                     *
 *              bcdesai@cs.concordia.ca                           *
 *              jababo@cs.concordia.ca                            *
 *                                                                *
 ******************************************************************/


/******************************************************************/
/***********************     maexport.c    ************************/
/******************************************************************/
/*                                                                */
/*    This File Contains all the functions needed to export class */
/*  definitions and class  instances to external files. The expo- */
/*  rted files are compatible with Postgres monitor command files */
/*  in that there is a \g at the end of  each command and this is */
/*  is useful in case those commands need to be issued to Postgres*/
/*  from other Postgres Tools than GDEP. It also contains fnctions*/
/*  to import a file containing Postquel commands to the Database.*/
/*                                                                */
/******************************************************************/

#include "mainfile.h"



/*****************************************************************/
/*                XaFfixReturnCharacterInPostquelFile            */
/*    Function to replace the \n character in a Postquel file by */
/* \\n, in the append queries of text fields only.               */
/*                                                               */
/*****************************************************************/

int XaFfixReturnCharacterInPostquelFile(XaLwidget, XaLpClientData)
  Widget  XaLwidget ;
  MaClientDataStruct XaLpClientData ;

  { /* begin XaFfixReturnCharacterInPostquelFile function */
    MaLabelType              MaLtempExportFilename      ;
    FILE                    *MaLtempStreamExportFile    ;

    int MaLrecordLength ;
    MaBufferType MaLrecord ;
    MaBufferType MaLinputRecord ;
    MaBufferType MaLprintRecord ;

    extern char* XaFsetCarriageReturn() ;

    strcpy(MaLtempExportFilename, XaLpClientData -> MaLexportFilename) ;
    strcat(MaLtempExportFilename, ".TEMP") ;

    if ((MaLtempStreamExportFile = fopen(MaLtempExportFilename, "w")) == NULL)
      {
        printf("Warning !!! Unable to open file %s for write mode\n",
               MaLtempExportFilename) ;
        return(MaFALSE) ;
      }

    if ((XaLpClientData -> MaLstreamExportFile =
           fopen(XaLpClientData -> MaLexportFilename, "r")) == NULL)
      {
        printf("Warning !!! Unable to open file %s for write mode\n",
               XaLpClientData -> MaLexportFilename) ;
        return(MaFALSE) ;
      }

    strcpy(MaLprintRecord, "\0") ;
    while (fgets (MaLrecord, MaMaxBufSize, XaLpClientData -> MaLstreamExportFile) 
            != NULL)
      {
        MaLrecordLength = strlen(MaLrecord) ;
        MaLrecord[MaLrecordLength - 1] = '\0' ;
        strcat(MaLprintRecord, MaLrecord) ;
        if (strcmp(&(MaLrecord[MaLrecordLength - 3]), "\\g") == 0)
          {
            /* printf("Record has slash g :: %s\n",MaLrecord) ; */
            strcat(MaLprintRecord, "\n") ;
            fprintf(MaLtempStreamExportFile, MaLprintRecord) ;
            strcpy(MaLprintRecord, "\0") ;
          }
         else
          {
            /* printf("Record does not have slash g :: %s\n",MaLrecord) ; */
            strcat(MaLprintRecord, "\\\\n") ;
          }

      }

    fclose(XaLpClientData -> MaLstreamExportFile) ;
    fclose(MaLtempStreamExportFile) ;

    sprintf(MaLrecord, "mv %s %s", MaLtempExportFilename,
                                   XaLpClientData -> MaLexportFilename) ;
    system(MaLrecord) ;

    return(MaTRUE) ;

  } /* end   XaFfixReturnCharacterInPostquelFile function */


/*****************************************************************/
/*                     XaFexportClassDefinition                  */
/*    Function to export the definition of a given class to a    */
/*  given filename.                                              */
/*                                                               */
/*****************************************************************/

int XaFexportClassDefinition(XaLwidget, XaLpClientData)
  Widget  XaLwidget ;
  MaClientDataStruct XaLpClientData ;

  { /* begin XaFexportClassDefinition function */

    extern MaStringListStruct MaFfreeStringList () ;
    extern MaClientDataStruct MaFgetPtrClientData() ;

    /**************************************************/
    /*  Retrieve the Class Attributes and their types */
    /**************************************************/
    if (!XaFsetClassAttributeNameType(XaLwidget, XaLpClientData))
      {
        sprintf(XaLpClientData -> MaLdialogMessage, 
               "Unable to get Data Type for Class %s",
               XaLpClientData -> MaLclassName) ;

        XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;
        return(MaFALSE) ;
      }

    /*********************************************************/
    /*  Set the Attribute type as the value of the attribute */
    /*********************************************************/
    {
      MaStringType              MaLstring ;
      MaBufferType              MaLquery  ;
      MaStringListStruct        XaLpStringList = NULL ;
      MaStringListStruct        XaLparentStringList = NULL ;
      extern MaStringListStruct MaFpointerNodeToListAdd() ;

      XaLpClientData -> MaLstringList = MaFpointerNodeToListAdd(
                                           XaLpClientData -> MaLtextStringList,
                                           XaLpClientData -> MaLstringList) ;

      XaLpClientData -> MaLtextStringList = NULL ;
      if (XaLpClientData -> MaLstringList == NULL)
        {
          MaStringType XaLmessage ;
          sprintf(XaLpClientData -> MaLdialogMessage, 
               "Class %s Does Not Have Any Attribute",
               XaLpClientData -> MaLclassName) ;
          XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;

          return(MaFALSE) ;
        }

        XaLpStringList = XaLpClientData -> MaLstringList ;
        while (XaLpStringList != NULL)
          {
            strcpy(XaLpStringList -> MaLstringData,
                   XaLpStringList -> MaLlabelType) ;
            XaLpStringList = XaLpStringList -> MaLpNextString ;
          }


        /*******************************************************************/
        /*  Check if Class Has Parents then exclude its parents Attributes */
        /*******************************************************************/
        if (XaLpClientData -> MaLexportInheritMode)
          {
            extern MaStringListStruct MaFgetStringListExePqCommand() ;
            extern MaClientDataStruct MaFfreeClientData () ;

            /*********************************************************************/
            /*  Check if Class Has New Parents then Inherit the New Parent Class */
            /*********************************************************************/
            if (strcmp(XaLpClientData -> MaLnewParentName, "") != 0)
              {
                extern MaStringListStruct MaFpointerStringToListAdd() ;
                XaLparentStringList = MaFpointerStringToListAdd(
                              XaLpClientData -> MaLnewParentName, XaLparentStringList) ;
              }
            else
              {
                sprintf(MaLquery,"retrieve (parent = padre.relname) \
                                  from clase, padre in pg_class, hereda in pg_inherits \
                                  where hereda.inhrel = clase.oid \
                                        and hereda.inhparent = padre.oid \
                                        and clase.relname =  \"%s\"",
                                  XaLpClientData -> MaLclassName) ;

                XaLparentStringList = MaFgetStringListExePqCommand(MaLquery) ;
              }

            if (XaLparentStringList != NULL)
              {
                MaClientDataStruct XaLtempClientData = MaFgetPtrClientData() ;
                strcpy(XaLtempClientData -> MaLclassName,
                       XaLparentStringList -> MaLstringData) ;

                if (XaFsetClassAttributeNameType(XaLwidget, XaLtempClientData))
                  {
                    MaStringListStruct XaLtestStringList ;
                    MaStringListStruct XaLtempStringList ;

                    XaLtempClientData -> MaLstringList = MaFpointerNodeToListAdd(
                                                 XaLtempClientData -> MaLtextStringList,
                                                 XaLtempClientData -> MaLstringList) ;
                    XaLtempClientData -> MaLtextStringList = NULL ;

                    XaLtempStringList = XaLtempClientData -> MaLstringList ;
                    while (XaLtempStringList != NULL)
                      {
                        XaLtestStringList = XaLpClientData -> MaLstringList ;
                        while (XaLtestStringList != NULL)
                          {
                            if (strcmp(XaLtestStringList -> MaLlabelName,
                                       XaLtempStringList -> MaLlabelName) == 0)
                              strcpy(XaLtestStringList -> MaLlabelName, "") ;

                            XaLtestStringList = XaLtestStringList -> MaLpNextString ;
                          }
                        XaLtempStringList = XaLtempStringList -> MaLpNextString ;
                      }
                  }
                XaLtempClientData = MaFfreeClientData (XaLtempClientData) ;
              }
          }


      /**************************************************/
      /*  Form the query to export the Class Definition */
      /**************************************************/

      sprintf(MaLstring, "create %s ", XaLpClientData -> MaLclassName) ;

      XaLpClientData -> MaLselectionAction =
                                 MaSelectionActionExportClassDefinition ;
      strcat (MaLstring,
              XaFsetNameValuesInstanceDisplayShell(XaLwidget, XaLpClientData)) ;


      /**************************************************************************/
      /*  If Class Inherits from his parent then add inherits Parent Class Name */
      /**************************************************************************/
      if (XaLparentStringList != NULL)
        {
          strcat(MaLstring, " inherits (") ;
          strcat(MaLstring, XaLparentStringList -> MaLstringData) ;
          strcat(MaLstring, ")") ;
          XaLparentStringList = MaFfreeStringList(XaLparentStringList) ;
        }


      strcat(MaLstring, MaClassExportSeparator) ;
      fprintf(XaLpClientData -> MaLstreamExportFile, MaLstring) ;
      sprintf(XaLpClientData -> MaLdialogMessage, 
              "Class Has Been Exported to File %s",
              XaLpClientData -> MaLexportFilename) ;

      XaLpClientData -> MaLdialogType = XaDialogNotice ;

    }
    return(MaTRUE) ;

  } /* end   XaFexportClassDefinition function */


/*****************************************************************/
/*                     XaFexportClassInstances                   */
/*    Function to export all instances of a given class to a     */
/*  given filename.                                              */
/*                                                               */
/*****************************************************************/

int XaFexportClassInstances(XaLwidget, XaLpClientData)
  Widget             XaLwidget ;
  MaClientDataStruct XaLpClientData ;

  { /* begin XaFhandleExportClassInstances function */

    MaStringType        MaLstring ;
    MaBufferType        MaLquery  ;

    MaStringListStruct XaLpCurrentOid ;

    extern MaStringListStruct MaFpointerNodeToListAdd() ;
    extern MaStringListStruct MaFgetStringListExePqCommand() ;


    /***************************************************/
    /*  Retrieve the oid of the Instances of the Class */
    /* and set the previous, current and next Oids.    */
    /***************************************************/
    XaLpClientData -> MaLfreeNodeOk = MaFALSE ;

    if (!XaFsetClassObjectIdsInstances(XaLwidget, XaLpClientData))
      {
        sprintf(XaLpClientData -> MaLdialogMessage, 
               "Class %s Does Not Have Any Instance",
               XaLpClientData -> MaLclassName) ;

        XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;
        return(MaFALSE) ;
      }

    XaLpClientData -> MaLfreeNodeOk = MaTRUE ;


    /**************************************************/
    /*  Retrieve the Class Attributes and their types */
    /**************************************************/
    if (!XaFsetClassAttributeNameType(XaLwidget, XaLpClientData))
      {
        sprintf(XaLpClientData -> MaLdialogMessage, 
               "Unable to get Data Type for Class %s",
               XaLpClientData -> MaLclassName) ;

        XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;
        return(MaFALSE) ;
      }


    /***************************************/
    /*  Set All The Attributes in one List */
    /***************************************/
    XaLpClientData -> MaLstringList = MaFpointerNodeToListAdd(
                                         XaLpClientData -> MaLtextStringList,
                                         XaLpClientData -> MaLstringList) ;

    XaLpClientData -> MaLtextStringList = NULL ;
    if (XaLpClientData -> MaLstringList == NULL)
      {
        sprintf(XaLpClientData -> MaLdialogMessage, 
               "Class %s Does Not Have Any Attribute",
               XaLpClientData -> MaLclassName) ;

        XaLpClientData -> MaLdialogType = XmDIALOG_ERROR ;
        return(MaFALSE) ;
      }


    /*********************************************************/
    /*  Set the Attribute type as the value of the attribute */
    /*********************************************************/
    XaLpCurrentOid = XaLpClientData -> MaLobjectIdsList ;

    while (XaLpCurrentOid != NULL)
     {
       XaLpClientData -> MaLcurrentOid = stoi(XaLpCurrentOid -> MaLstringData) ;
       MaFsetOidsInstanceAttributesValue(XaLpClientData) ;

      /**************************************************/
      /*  Form the query to export the Class Definition */
      /**************************************************/

      sprintf(MaLstring, "append %s ", XaLpClientData -> MaLclassName) ;

      XaLpClientData -> MaLselectionAction =
                                 MaSelectionActionExportClassInstances ;
      strcat (MaLstring,
              XaFsetNameValuesInstanceDisplayShell(XaLwidget, XaLpClientData)) ;

       strcat(MaLstring, MaClassExportSeparator) ;

       fprintf(XaLpClientData -> MaLstreamExportFile, MaLstring) ;

       XaLpCurrentOid = XaLpCurrentOid -> MaLpNextString ;

     } /* end while XaLpCurrentOid Loop */

    return(MaTRUE) ;

  } /* end   XaFexportClassInstances function */


/*****************************************************************/
/*                        XaFexportClassTree                     */
/*    Function to export all the definition and the instances of */
/*  all the classes that exist in the tree  hierarchy of a given */
/*  class. The output is written to a given File.                */
/*                                                               */
/*****************************************************************/

MaStringListStruct XaFexportClassTree(XaLwidget,XaLpClientData)
  Widget             XaLwidget ;
  MaClientDataStruct XaLpClientData ;

  { /* begin XaFexportClassTree function */

    MaStringType XaLquery ;

    MaStringListStruct XaLpClassTreeList    = NULL ;
    MaStringListStruct XaLtempClassTreeList = NULL ;

    extern MaStringListStruct MaFfreeStringList ()        ;
    extern MaStringListStruct MaFpointerNodeToListAdd()   ;
    extern MaStringListStruct MaFpointerStringToListAdd() ;
    extern MaClientDataStruct MaFfreeClientData ()  ;
    extern MaClientDataStruct MaFgetPtrClientData() ;


    /***********************************************************/
    /*  Form The List of Classes in the Tree of the Root Class */
    /***********************************************************/

    XaLpClassTreeList = MaFpointerStringToListAdd
                            (XaLpClientData -> MaLclassName, XaLpClassTreeList) ;

    XaLtempClassTreeList = XaLpClassTreeList ;
    while (XaLtempClassTreeList != NULL)
      {
        MaStringListStruct XaLloop1TreeList    = NULL ;
        MaStringListStruct XaLloop2TreeList    = NULL ;
        MaStringListStruct XaLchildrenList     = NULL ;
        MaStringListStruct XaLtempChildrenList = NULL ;
        
        sprintf(XaLquery,"retrieve (child=hijo.relname) \
                         from  clase, hijo in pg_class, hereda in pg_inherits \
                         where hereda.inhparent = clase.oid \
                               and hereda.inhrel = hijo.oid \
                               and clase.relname =  \"%s\"",
                XaLtempClassTreeList -> MaLstringData) ;

        XaLchildrenList = MaFgetStringListExePqCommand(XaLquery) ;
        while(XaLchildrenList != NULL)
          {
            XaLtempChildrenList = XaLchildrenList ;
            XaLchildrenList = XaLchildrenList -> MaLpNextString ;
            XaLtempChildrenList -> MaLpNextString = NULL ;
            if (!(MaFwordInStringList(XaLtempChildrenList -> MaLstringData,
                                    XaLpClassTreeList)))
              XaLtempClassTreeList = MaFpointerNodeToListAdd(XaLtempChildrenList,
                                                             XaLtempClassTreeList) ;
            else
              XaLtempChildrenList = MaFfreeStringList(XaLtempChildrenList) ;
          }

        XaLtempClassTreeList = XaLtempClassTreeList -> MaLpNextString ;
      }


    /*************************************************************************/
    /*  Export Class Definition & Class Instances of Every Class in the tree */
    /*************************************************************************/
    XaLtempClassTreeList = XaLpClassTreeList ;
    while (XaLtempClassTreeList != NULL)
      {
        strcpy(XaLpClientData -> MaLclassName, XaLtempClassTreeList -> MaLstringData) ;

        XaLpClientData -> MaLobjectIdsList  =
                       MaFfreeStringList(XaLpClientData -> MaLobjectIdsList) ;
        XaLpClientData -> MaLstringList     =
                        MaFfreeStringList(XaLpClientData -> MaLstringList) ;
        XaLpClientData -> MaLtextStringList =
                        MaFfreeStringList(XaLpClientData -> MaLtextStringList) ;
        (void) XaFexportClassDefinition(XaLwidget, XaLpClientData) ;

        XaLpClientData -> MaLobjectIdsList  =
                       MaFfreeStringList(XaLpClientData -> MaLobjectIdsList) ;
        XaLpClientData -> MaLstringList     =
                        MaFfreeStringList(XaLpClientData -> MaLstringList) ;
        XaLpClientData -> MaLtextStringList =
                        MaFfreeStringList(XaLpClientData -> MaLtextStringList) ;
        (void) XaFexportClassInstances (XaLwidget, XaLpClientData)  ;

        XaLtempClassTreeList = XaLtempClassTreeList -> MaLpNextString ;
      }

    return(XaLpClassTreeList) ;

  } /* end   XaFexportClassTree function */


/*****************************************************************/
/*                      XaFimportPostQuelFile                    */
/*    Function to import to the Database the Postquel Commands   */
/*  of a specified file.                                         */
/*                                                               */
/*****************************************************************/

void XaFimportPostQuelFile(XaLwidget,XaLpClientData)
  Widget             XaLwidget ;
  MaClientDataStruct XaLpClientData ;

  { /* begin XaFimportPostQuelFile function */
    int XaLindex ;
    int XaLrecordLength ;
    int XaLnotFound = MaTRUE ;
    int XaLlength = strlen(MaClassExportSeparator) ;
    MaBufferType XaLquery ;

    while(fgets(XaLquery, MaMaxBufSize,
                XaLpClientData -> MaLstreamExportFile) != NULL) 
      {

        XaLrecordLength = strlen(XaLquery) ;
        XaLquery[XaLrecordLength - 3] = '\0' ;
        XaLrecordLength = XaLrecordLength - 3 ;

        for (XaLindex = 0; XaLindex < XaLrecordLength; XaLindex++)
          if (strncmp(&(XaLquery[XaLindex]),"\\n",2) == 0)
            {
              XaLquery[XaLindex - 1] = ' ' ;
              XaLindex++ ;
            }

        PQexec("begin") ;
        PQexec(XaLquery) ;
        PQexec("end") ;

      } /* end while fgets */

  } /* end   XaFimportPostQuelFile function */


/*****************************************************************/
/*                                                               */
/*                       END OF maexport.c FILE                  */
/*                                                               */
/*****************************************************************/
